"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.usePropertyProxy = void 0;
const util_1 = require("./util");
const zod_1 = require("zod");
const class_validator_1 = require("class-validator");
const ValidationError_1 = require("./Error/ValidationError");
const yup_1 = require("yup");
function usePropertyProxy() {
    let schemaObj;
    let schemaTarget;
    function Validate(schama) {
        return function (target) {
            return new Proxy(target, {
                construct: (_target, args) => {
                    if (schama) {
                        schemaObj = schama;
                    }
                    schemaTarget = new _target(...args);
                    return new target(...args);
                }
            });
        };
    }
    function Property() {
        return function (target, key) {
            Object.defineProperty(target, key, {
                get: () => "",
                set: function (v) {
                    let val;
                    Object.defineProperty(this, key, {
                        get: () => { var _a, _b; return ((_b = (_a = ObjectProxy[key]) === null || _a === void 0 ? void 0 : _a.getter) === null || _b === void 0 ? void 0 : _b.call(_a, val)) || val; },
                        set: (newValue) => {
                            var _a, _b, _c;
                            if ((_a = ObjectProxy[key]) === null || _a === void 0 ? void 0 : _a.setter) {
                                (_c = (_b = ObjectProxy[key]).setter) === null || _c === void 0 ? void 0 : _c.call(_b, { setValue(value) {
                                        val = value;
                                    }, value: newValue });
                                return;
                            }
                            val = newValue;
                            if (schemaTarget && schemaTarget[key] !== newValue) {
                                Object.assign(schemaTarget, { [key]: newValue });
                            }
                            // start support class-validator
                            if (schemaTarget) {
                                (0, class_validator_1.validate)(schemaTarget).then(errors => {
                                    if (errors.length > 0) {
                                        const errorsMaped = errors.map(error => {
                                            return {
                                                property: error.property,
                                                message: (error.constraints && Object.values(error.constraints)[0]),
                                                type: error.constraints && Object.keys(error.constraints)[0]
                                            };
                                        });
                                        throw new ValidationError_1.ValidationError(errorsMaped);
                                    }
                                });
                            }
                            // end support class-validator
                            // start support zod
                            if (schemaObj && schemaTarget && schemaObj instanceof zod_1.ZodObject) {
                                try {
                                    schemaObj.parse(schemaTarget);
                                }
                                catch (_error) {
                                    const error = _error;
                                    const errorsMaped = error.errors.map(error => {
                                        return {
                                            property: error.path[0].toString(),
                                            message: error.message,
                                            type: error.code
                                        };
                                    });
                                    throw new ValidationError_1.ValidationError(errorsMaped);
                                }
                            }
                            // end support zod
                            // start support yup
                            if (schemaObj && schemaTarget && schemaObj instanceof yup_1.ObjectSchema) {
                                schemaObj.validate(schemaTarget, { abortEarly: false }).catch(_error => {
                                    const error = _error;
                                    const errorsMaped = error.inner.map(error => {
                                        return {
                                            property: error.path,
                                            message: error.errors[0],
                                            type: error.type
                                        };
                                    });
                                    throw new ValidationError_1.ValidationError(errorsMaped);
                                });
                            }
                            // end support yup
                        },
                        enumerable: true,
                    });
                    this[key] = v;
                },
            });
        };
    }
    const ObjectProxy = {};
    const proxy = new Proxy({}, {
        get(_target, p) {
            const isSet = p.includes("set");
            const originalParam = isSet ? (0, util_1.lowCaseFirstLetter)(p.split("set")[1]) : (0, util_1.lowCaseFirstLetter)(p.split("get")[1]);
            const obj = {
                [p]: (cb) => {
                    Object.assign(ObjectProxy, {
                        [originalParam]: {
                            [`${isSet ? "setter" : "getter"}`]: cb
                        }
                    });
                }
            };
            return obj[p];
        },
    });
    return { Property, proxy, Validate };
}
exports.usePropertyProxy = usePropertyProxy;
